<?php

/*
Plugin Name: CDNJS for WordPress
Plugin URI: http://worpit.com/
Description: Allows you to easily include Javascript and other resources from CDNJS.
Version: 1.2.b
Author: Worpit
Author URI: http://worpit.com/
*/

/**
 * Copyright (c) 2013 Worpit <support@worpit.com>
 * All rights reserved.
 *
 * "CDNJS for WordPress" is
 * distributed under the GNU General Public License, Version 2,
 * June 1991. Copyright (C) 1989, 1991 Free Software Foundation, Inc., 51 Franklin
 * St, Fifth Floor, Boston, MA 02110, USA
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

require_once( dirname(__FILE__).'/src/worpit-plugins-base.php' );
include_once( dirname(__FILE__).'/hlt-rssfeed-widget.php' );

function _worpit_cdnjs_e( $insStr ) {
	_e( $insStr, 'worpit-cdnjs' );
}
function _worpit_cdnjs__( $insStr ) {
	return __( $insStr, 'worpit-cdnjs' );
}

if ( !class_exists('Worpit_Cdnjs_Main') ):

class Worpit_Cdnjs_Main extends Worpit_Cdnjs_Base_Plugin {
	
	const InputPrefix				= 'worpit_cdnjs_';
	const OptionPrefix				= 'worpit_cdnjs_'; //ALL database options use this as the prefix.

	static public $VERSION			= '1.2.b'; //SHOULD BE UPDATED UPON EACH NEW RELEASE
	
	static public $aWP_MAPPINGS;
	static public $sALL_PACKAGES_URL = 'http://cdnjs.com/packages.json';
	static public $oPACKAGES_CONTENT;
	
	protected $m_aAllPluginOptions;
	protected $m_aPluginOptions_CdnJs_JsIncludes;
	protected $m_aPluginOptions_CdnJs_CssIncludes;
	protected $m_aPluginOptions_GeneralSection;
	
	public function __construct() {
		parent::__construct();

		register_activation_hook( __FILE__, array( &$this, 'onWpActivatePlugin' ) );
		register_deactivation_hook( __FILE__, array( &$this, 'onWpDeactivatePlugin' ) );
	//	register_uninstall_hook( __FILE__, array( &$this, 'onWpUninstallPlugin' ) );
		
		self::$PLUGIN_NAME	= basename(__FILE__);
		self::$PLUGIN_PATH	= plugin_basename( dirname(__FILE__) );
		self::$PLUGIN_DIR	= WP_PLUGIN_DIR.WORPIT_DS.self::$PLUGIN_PATH.WORPIT_DS;
		self::$PLUGIN_URL	= plugins_url( '/', __FILE__ ) ; //this seems to use SSL more reliably than WP_PLUGIN_URL
		self::$OPTION_PREFIX = self::OptionPrefix;
		
		$this->m_sPluginSlug = 'cdnjs';
		
		if ( !is_admin() ) {
			add_action( 'init', array( &$this, 'enqueueAll' ), 999 );
		}
		
	}//__construct
	
	protected function initPluginOptions() {
		
		$this->m_aPluginOptions_CdnJs_JsIncludes = array(
			'section_title' => 'Choose Common Javascript Libraries'
		);
		$this->m_aPluginOptions_CdnJs_CssIncludes = array(
			'section_title' => 'Choose Common CSS Libraries'
		);

		self::$oPACKAGES_CONTENT = file_get_contents( self::$sALL_PACKAGES_URL );
		
		if ( self::$oPACKAGES_CONTENT !== false && function_exists( 'json_decode' ) ) {
			$this->readAllPackages();
		}
		else {
			$this->m_aPluginOptions_CdnJs_JsIncludes['section_options'] = array(
				array(	'cufon',				'',		'N', 	'checkbox',	'Cufón', 'Include the latest Cufón JS library', "" ),
				array(	'firebug-lite',			'',		'N', 	'checkbox',	'Firebug Lite', 'Include the Firebug Lite JS library', "" ),
				array(	'jquery',				'',		'N', 	'checkbox',	'JQuery', 'Include the latest JQuery library', "Will replace your WordPress JQuery library." ),
				array(	'jqueryui',				'',		'N', 	'checkbox',	'JQuery UI', 'Include the latest JQuery UI library', "" ),
				array(	'json2',				'',		'N', 	'checkbox',	'JSON 2', 'Include the latest JSON 2 library', "" ),
				array(	'json3',				'',		'N', 	'checkbox',	'JSON 3', 'Include the latest JSON 3 library', "" ),
				array(	'less',					'',		'N', 	'checkbox',	'less.js', 'Include the latest LESS Compiler JS library', "" ),
				array(	'mootools',				'',		'N', 	'checkbox',	'Mootools', 'Include the latest Mootools library', "" ),
				array(	'prettify',				'',		'N', 	'checkbox',	'Prettify', 'Include the latest Prettify library', "" ),
				array(	'twitter-bootstrap',	'',		'N', 	'checkbox',	'Twitter Bootstrap', 'Include the latest Twitter Bootstrap JS library', "Remember to include the CSS below" ),
			);
			$this->m_aPluginOptions_CdnJs_CssIncludes['section_options'] = array(
				array(	'960gs',					'',	'N', 	'checkbox',	'960gs', 'Include the latest 960gs CSS library', "" ),
				array(	'normalize',				'',	'N', 	'checkbox',	'Normalize', 'Include the latest Normalize library', "" ),
				array(	'font-awesome',				'',	'N', 	'checkbox',	'Font Awesome', 'Include the latest Font Awesome library', "" ),
			);
			$this->m_aPluginOptions_GeneralSection = array(
				'section_title' => 'General CDNJS Plugin Options',
				'section_options' => array( )
			);
			
		}//if-else
		
		$this->m_aAllPluginOptions = array(
			&$this->m_aPluginOptions_CdnJs_CssIncludes,
			&$this->m_aPluginOptions_CdnJs_JsIncludes,
//			&$this->m_aPluginOptions_GeneralSection
		);
		
		/**
		 * This allows us to replace the WordPress libraries. e.g. CDNJS has jqueryui, but this is registered in WP
		 * as jquery-ui.
		 */
		self::$aWP_MAPPINGS = array(
			'jqueryui'	=>	'jquery-ui'
		);
		
		return true;
		
	}//initPluginOptions

	public function onWpInit() {
		parent::onWpInit();

//		add_action( 'wp_enqueue_scripts', array( &$this, 'onWpPrintStyles' ) );
//		add_action( 'wp_enqueue_scripts', array( &$this, 'onWpEnqueueScripts' ) );
		
	}//onWpInit
	
	public function onWpAdminInit() {
		parent::onWpAdminInit();
		
		global $pagenow;
		//Loads the news widget on the Dashboard (if it hasn't been disabled)
		if ( $pagenow == 'index.php' ) {
			$sDashboardRssOption = self::getOption( 'hide_dashboard_rss_feed' );
			if ( empty( $sDashboardRssOption ) || self::getOption( 'hide_dashboard_rss_feed' ) == 'N' ) {
				
				if ( class_exists(HLT_DashboardRssWidget) ) {
					$oHLT_DashboardRssWidget = new HLT_DashboardRssWidget();
				}
			}
		}
		
		//Enqueues the WP Admin Twitter Bootstrap files if the option is set or we're in a Worpit admin page.
		if ( $this->isWorpitPluginAdminPage() ) {
			add_action( 'admin_enqueue_scripts', array( &$this, 'enqueueBootstrapAdminCss' ), 99 );
		}
		
		//Multilingual support.
		load_plugin_textdomain( 'worpit-cdnjs', false, basename( dirname( __FILE__ ) ) . '/languages' );
	}
	
	protected function createPluginSubMenuItems(){
		$this->m_aPluginMenu = array(
			//Menu Page Title => Menu Item name, page ID (slug), callback function for this page - i.e. what to do/load.
			$this->getSubmenuPageTitle( 'CDNJS' ) => array( 'Includes', $this->getSubmenuId( 'includes' ), 'onDisplayCdnjsIncludes' ),
		);
	}//createPluginSubMenuItems
	
	protected function handlePluginUpgrade() {
		
		//current_user_can( 'manage_options' ) ensure only valid users attempt this.
		if ( self::getOption( 'current_plugin_version' ) !== self::$VERSION && current_user_can( 'manage_options' ) ) {

			//Set the flag so that this update handler isn't run again for this version.
			self::updateOption( 'current_plugin_version', self::$VERSION );
		}//if

		//Someone clicked the button to acknowledge the update
		if ( isset( $_POST['hlt_hide_update_notice'] ) && isset( $_POST['hlt_user_id'] ) ) {
			$result = update_user_meta( $_POST['hlt_user_id'], self::OptionPrefix.'current_version', self::$VERSION );
			header( "Location: admin.php?page=".$this->getFullParentMenuId() );
		}
		
	}//handlePluginUpgrade
	
	public function onWpAdminNotices() {
		
		//Do we have admin priviledges?
		if ( !current_user_can( 'manage_options' ) ) {
			return;
		}
		
		$this->adminNoticeVersionUpgrade();
		$this->adminNoticeOptionsUpdated();
	}
	
	private function adminNoticeVersionUpgrade() {

		global $current_user;
		$user_id = $current_user->ID;

		$sCurrentVersion = get_user_meta( $user_id, self::OptionPrefix.'current_version', true );

		if ( $sCurrentVersion !== self::$VERSION ) {
			$sNotice = '
					<style>
						a#fromWorpit {
							padding: 0 5px;
							border-bottom: 1px dashed rgba(0,0,0,0.1);
							color: black;
						}
					</style>
					<form method="post" action="admin.php?page=worpit-cdnjs">
						<p><strong>CDNJS Plugin, <a href="http://bit.ly/QhYJzY" id="fromWorpit" title="Manage WordPress Better" target="_blank">from Worpit</a></strong> has been updated successfully.
						<input type="hidden" value="1" name="hlt_hide_update_notice" id="hlt_hide_update_notice">
						<input type="hidden" value="'.$user_id.'" name="hlt_user_id" id="hlt_user_id">
						<input type="submit" value="Okay, take me to the main plugin page and hide this notice" name="submit" class="button-primary">
						</p>
					</form>
			';

			$this->getAdminNotice( $sNotice, 'updated', true );
		}
		
	}//adminNoticeVersionUpgrade
	
	private function adminNoticeOptionsUpdated() {
		
		$sAdminFeedbackNotice = $this->getOption( 'feedback_admin_notice' );
		if ( !empty( $sAdminFeedbackNotice ) ) {
			$sNotice = '<p>'.$sAdminFeedbackNotice.'</p>';
			$this->getAdminNotice( $sNotice, 'updated', true );
			$this->updateOption( 'feedback_admin_notice', '' );
		}
		
	}//adminNoticeOptionsUpdated
	
	public function onDisplayCdnjsIncludes() {
		
		//populates plugin options with existing configuration
		$aAllIncludes = array();
		$aIncludes = $this->getOption( 'includes' );
		if ( $aIncludes !== false ) {
			foreach( $aIncludes as $sInclude => $aData ) {
				list( $sVersion, $sFilename ) = $aData;
				$sComplete = implode( '_SEP_', array( self::$OPTION_PREFIX.$sInclude, $sVersion, $sFilename ) );
				$sComplete = str_replace( '.', '_DOT_', $sComplete);
				$aAllIncludes[ $sComplete ] = 'Y';
			}
		}
		$this->populateAllPluginOptions( $aAllIncludes );

		//Specify what set of options are available for this page
		$aAvailableOptions = array(
			&$this->m_aPluginOptions_CdnJs_CssIncludes,
			&$this->m_aPluginOptions_CdnJs_JsIncludes
		);
		
		$aData = array(
			'plugin_url'		=> self::$PLUGIN_URL,
			'var_prefix'		=> self::OptionPrefix,
			'aAllOptions'		=> $aAvailableOptions,
			'nonce_field'		=> $this->getSubmenuId( 'includes' ),
			'form_action'		=> 'admin.php?page='.$this->getSubmenuId( 'includes' )
		);

		$this->display( $this->getSubmenuId('includes'), $aData );
		
	}//onDisplayCdnjsIncludes
	
	protected function readAllPackages() {
		
		$oAllPackages = json_decode( self::$oPACKAGES_CONTENT );
		$oAllPackages = $oAllPackages->packages;

		$this->m_aPluginOptions_CdnJs_JsIncludes['section_options'] = array();
		$this->m_aPluginOptions_CdnJs_CssIncludes['section_options'] = array();
		foreach( $oAllPackages as $oPackage ) {
			
			if ( substr($oPackage->filename, -3) == '.js'  ) {
				
				$this->m_aPluginOptions_CdnJs_JsIncludes['section_options'][] = array(
					str_replace( '.', '_DOT_', $oPackage->name.'_SEP_'.$oPackage->version.'_SEP_'.$oPackage->filename ),
					'', 'N', 'checkbox',
					$oPackage->name,
					'Add the '.$oPackage->name.' JS Library',
					esc_attr( $oPackage->description ),
					$oPackage->version
				);
			}
			elseif ( substr($oPackage->filename, -4) == '.css' ) {
				$this->m_aPluginOptions_CdnJs_CssIncludes['section_options'][] = array(
					str_replace( '.', '_DOT_', $oPackage->name.'_SEP_'.$oPackage->version.'_SEP_'.$oPackage->filename ),
					'', 'N', 'checkbox',
					$oPackage->name,
					'Add the '.$oPackage->name.' CSS Library',
					esc_attr( $oPackage->description ),
					$oPackage->version
				);
			}
		}
		
		//Not entirely necessary, but makes the interface a little better
		sort( $this->m_aPluginOptions_CdnJs_CssIncludes['section_options'] );
		sort( $this->m_aPluginOptions_CdnJs_JsIncludes['section_options'] );
	}
	
	protected function handlePluginFormSubmit() {
		
		if ( !isset( $_POST['worpit_plugin_form_submit'] ) ) {
			return;
		}
		
		$this->m_fSubmitCbcMainAttempt = true;
	
		if ( isset( $_GET['page'] ) ) {
			switch ( $_GET['page'] ) {
				case $this->getSubmenuId( 'includes' ):
					$this->handleSubmit_CdnjsIncludes();
					break;
			}
		}
		
		if ( !self::$m_fUpdateSuccessTracker ) {
			self::updateOption( 'feedback_admin_notice', 'Updating Settings <strong>Failed</strong>.' );
		}
		else {
			self::updateOption( 'feedback_admin_notice', 'Updating Settings <strong>Succeeded</strong>.' );
		}
		
	}//handlePluginFormSubmit
	
	protected function handleSubmit_CdnjsIncludes() {

		//Ensures we're actually getting this request from WP.
		check_admin_referer( $this->getSubmenuId('includes') );
		
		// Will store values serialized
		// First remove unnecessary POST values.
		unset( $_POST['_wpnonce'] );
		unset( $_POST['_wp_http_referer'] );
		unset( $_POST['worpit_plugin_form_submit'] );
		unset( $_POST['submit'] );
		unset( $_POST[self::OptionPrefix.'all_options_input'] );
		
		$aOptions = array();
		foreach( $_POST as $sIncludeName => $val ) {
			$sIncludeName = str_replace( self::OptionPrefix, '', $sIncludeName );
			$sIncludeName = str_replace( '_DOT_', '.', $sIncludeName);
			list($sName, $sVersion, $sFileName) = explode( '_SEP_', $sIncludeName );
			$aOptions[ $sName ] = array( $sVersion, $sFileName );
		}

		self::updateOption( 'includes', $aOptions );

	}

	public function enqueueAll() {
		
		$aAllIncludes = $this->getOption( 'includes' );
		
		//No includes have been set yet.
		if ( empty($aAllIncludes) ) {
			return;
		}

		//Requires WordPress 3.5+
		$sUrlStem = '//cdnjs.cloudflare.com/ajax/libs/';
		
		foreach( $aAllIncludes as $sIncludeName => $aDetails ) {
			
			list( $sVersion, $sFileName ) = $aDetails;
			
			//Ensure we actually replace WordPress libraries.
			if ( isset(self::$aWP_MAPPINGS[ $sIncludeName ]) ) {
				$sWpName = self::$aWP_MAPPINGS[ $sIncludeName ];
			}
			else {
				$sWpName = $sIncludeName;
			}

			$this->enqueueResource( $sWpName, $sUrlStem.$sIncludeName.'/'.$sVersion.'/'.$sFileName, $sVersion );

			// if it is bootstrap, also enqueue the CSS
			if ( $sIncludeName == 'twitter-bootstrap' ) {
				$this->enqueueResource( $sIncludeName.rand(), $sUrlStem.$sIncludeName.'/'.$sVersion.'/css/bootstrap.min.css', $sVersion );
				$this->enqueueResource( $sIncludeName.rand(), $sUrlStem.$sIncludeName.'/'.$sVersion.'/css/bootstrap-responsive.min.css', $sVersion );
			}
		}
		
	}//onWpEnqueueScripts
	
	public function enqueueResource( $insResourceName, $insResourceSource, $insVersion ) {
		
		if ( substr($insResourceSource, -3) == '.js' ) {
			wp_deregister_script( $insResourceName );
			wp_register_script( $insResourceName, $insResourceSource, false, $insVersion );
			wp_enqueue_script( $insResourceName );
			
		}
		elseif ( substr($insResourceSource, -4) == '.css' ) {
			wp_deregister_style( $insResourceName );
			wp_register_style( $insResourceName, $insResourceSource, false, $insVersion );
			wp_enqueue_style( $insResourceName );
		}
	}//enqueueResource
	
	
	protected function checkUrlValid( $insUrl ) {
		$oCurl = curl_init();
		curl_setopt( $oCurl, CURLOPT_URL, $insUrl );
		curl_setopt( $oCurl, CURLOPT_RETURNTRANSFER, 1 );
		curl_setopt( $oCurl, CURLOPT_CONNECTTIMEOUT, 10 );
		
		$sContent = curl_exec( $oCurl );
		$sHttpCode = curl_getinfo( $oCurl, CURLINFO_HTTP_CODE );
		curl_close( $oCurl );
		
		return ( intval( $sHttpCode ) === 200 );
	}

	/**
	 * Not currently used, but could be useful once we work out what way the JS should be included.
	 * @param $insHandle	For example: 'prettify/prettify.css'
	 */
	protected function isRegistered( $insHandle ) {
		return (
			wp_script_is( $insHandle, 'registered' ) ||
			wp_script_is( $insHandle, 'queue' ) ||
			wp_script_is( $insHandle, 'done' ) ||
			wp_script_is( $insHandle, 'to_do' )
		);
	}
	
	public function onWpPluginActionLinks( $inaLinks, $insFile ) {
		if ( $insFile == plugin_basename( __FILE__ ) ) {
			$sSettingsLink = '<a href="'.admin_url( "admin.php" ).'?page='.$this->getSubmenuId('includes').'">' . _worpit_cdnjs__( 'Settings' ) . '</a>';
			array_unshift( $inaLinks, $sSettingsLink );
		}
		return $inaLinks;
	}
	
	protected function deleteAllPluginDbOptions() {
		
	//	parent::deleteAllPluginDbOptions();
		
		if ( !current_user_can( 'manage_options' ) ) {
			return;
		}
		
	}//deleteAllPluginDbOptions
	
	public function onWpDeactivatePlugin() {
		
		if ( $this->getOption('delete_on_deactivate') == 'Y' ) {
			$this->deleteAllPluginDbOptions();
		}
		
	//	$this->deleteOption( 'includes' );
		$this->deleteOption( 'current_plugin_version' );
		$this->deleteOption( 'feedback_admin_notice' );
		
	}//onWpDeactivatePlugin
	
	public function onWpActivatePlugin() { }
	
	public function enqueueBootstrapAdminCss() {
		wp_register_style( 'worpit_bootstrap_wpadmin_css', $this->getCssUrl('bootstrap-wpadmin.css'), false, self::$VERSION );
		wp_enqueue_style( 'worpit_bootstrap_wpadmin_css' );
		wp_register_style( 'worpit_bootstrap_wpadmin_css_fixes', $this->getCssUrl('bootstrap-wpadmin-fixes.css'),  array('worpit_bootstrap_wpadmin_css'), self::$VERSION );
		wp_enqueue_style( 'worpit_bootstrap_wpadmin_css_fixes' );
	}//enqueueBootstrapAdminCss
	
}//Worpit_Cdnjs_Main

endif;

$oWorpit_Cdnjs_Main = new Worpit_Cdnjs_Main();
